@@ -1,14 +1,13 @@
 #' @rdname extract_sample
 #' @name extract_sample
 #' @title Extract a sample from a \code{LibBi} run.
-#' @description This function takes the provided \code{\link{libbi}} 
-#'     object which has been run and extracts a data frame.
-#'     traces.
+#' @description This function takes the provided \code{\link{libbi}}
+#'     results and extracts a data frame.
 #' @param x a \code{\link{libbi}} object which has been run, or a
 #'     list of data frames containing parameter traces (as returned by
 #'     from \code{bi_read}); if it is not a \code{\link{libbi}}
 #'     object, either 'all' must be TRUE or a model given
-#' @param np iteration to extract; if not given a random sample will be extracted
+#' @param np iteration to extract; if set to "last", the last sample will be extracted. If not given a random sample will be extracted
 #' @param ... parameters to \code{bi_read} (e.g., dimensions)
 #' @return list of parameters and trajectories
 #' @export
@@ -32,7 +31,7 @@ extract_sample <- function(x, np, ...) {
     }
   }, 0))
 
-  find_np <- np
+  find_np <- ifelse(tolower(np) == "last", max_np, np)
   if (find_np > max_np) {
     stop("np requested greater than the maximum ", max_np)
   }
