@@ -1,10 +1,5 @@
 ### This demo shows how to perform filtering on a simple 
 ### synthetic dataset using libbi.
-rm(list = ls(all.names=TRUE))
-unlink(".RData")
-try(detach(package:rbi, unload = TRUE), silent = TRUE)
-library(rbi, quietly = TRUE)
-
 # the PZ model file is included in rbi and can be found there:
 model_file_name <- system.file(package="rbi", "PZ.bi")
 
@@ -19,13 +14,16 @@ init_parameters <- list(P = 2, Z = 2, mu = 0.5, sigma = 0.3)
 synthetic_dataset <- bi_generate_dataset(end_time=T, model=PZ,
                                          init=init_parameters)
 # Settings
-bi_object <- libbi$new(client="filter", model=PZ)
-print(bi_object)
+bi_object <- libbi(model=PZ)
+bi_object
 # Once happy with the settings, launch bi.
-bi_object$run(nparticles = 8192, nthreads = 1, end_time = T, noutputs = T, obs = synthetic_dataset, init = init_parameters)
+bi_object <- filter(bi_object, nparticles = 8192, nthreads = 1, end_time = T, noutputs = T, obs = synthetic_dataset, init = init_parameters)
 # It can be a good idea to look at the result file
 bi_file_summary(bi_object$output_file_name)
-bi_read(bi_object$output_file_name, vars = "mu")
+bi_object
+summary(bi_object)
+## read mu variable
+bi_read(bi_object, vars = "mu")
 # Let's have a look at the filtering means
 # First, get the particles
 output <- bi_read(bi_object)
