@@ -2,7 +2,13 @@ context("Testing saving and retrieving libbi objects")
 
 bi <- libbi(model = system.file(package="rbi", "PZ.bi"))
 example_output_file <- system.file(package="rbi", "example_output.nc")
-bi <- add_output(bi, example_output_file)
+bi <- attach_file(bi, "output", example_output_file)
+
+test_that("we can't add output twice'",
+{
+  expect_error(attach_file(bi, "output", example_output_file))
+  expect_equal(class(attach_file(bi, "output", example_output_file, force=TRUE)), "libbi")
+})
 
 test_that("libbi object with added output is functional",
 {
